@@ -46,8 +46,7 @@ namespace internal {
         explicit average_pooling_3d_layer(const std::string& name,
             const shape3& pool_size, const shape3& strides,
             padding p)
-            : pooling_3d_layer(name, pool_size, strides, p,
-                &inner_average_pool)
+            : pooling_3d_layer(name, pool_size, strides, p, &inner_average_pool)
         {
         }
     };
